@@ -1,6 +1,6 @@
 /*
  * Strsed.c
- * $Id: strsed.c 19376 2006-09-02 03:19:06Z yeled@macports.org $
+ * $Id: strsed.c 112417 2013-10-22 13:21:29Z jmr@macports.org $
  *
  *     ed(1)/tr(1)-like search, replace, transliterate. See the
  *     manpage for details. See the README for copyright information.
@@ -129,6 +129,8 @@
 
 #define HS_REGEX	1
 
+#include "strsed.h"
+
 #include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
@@ -215,13 +217,14 @@ static struct {
 /* ------------------------------------------------------------------------- **
  * Prototypes
  * ------------------------------------------------------------------------- */
-static char *mem();
-static void mem_init();
-static void mem_free();
-static char *build_map();
-static char nextch();
-static void mem_save();
-static int mem_find();
+static char *mem(int, int);
+static void mem_init(void);
+static void mem_free(char *);
+static char *build_map(char *, char *);
+static char nextch(char *, int);
+static void mem_save(int);
+static int mem_find(int);
+char *backslash_eliminate(char *, int, int);
 
 /* ------------------------------------------------------------------------- **
  * strsed
@@ -245,8 +248,6 @@ int *range;
     static regmatch_t *exp_regs = NULL;
     static regex_t exp;
 #endif
-
-    char *backslash_eliminate();
     
     char *from;
     char *new_str;
@@ -591,6 +592,12 @@ int *range;
      */
 
     do {
+#ifdef HS_REGEX
+	    /* XXX Not even trying to use custom memory routines */
+	    if (!(exp_regs = calloc(str_len, sizeof(regmatch_t)))) {
+		return 0;
+	    }
+#endif
 	if (match_all){
 	    /* Fake a match instead of calling re_search() or regexec(). */
 	    match = 1;
@@ -604,10 +611,6 @@ int *range;
 	    match = re_search(&re_comp_buf, str, str_len, 0, str_len, &regs);
 #endif
 #ifdef HS_REGEX
-	    /* XXX Not even trying to use custom memory routines */
-	    if (!(exp_regs = calloc(str_len, sizeof(regmatch_t)))) {
-		return 0;
-	    }
 	    match = regexec(&exp, str, str_len, exp_regs, 0) ? NO_MATCH : 1;
 #endif
 	}
@@ -659,7 +662,7 @@ int *range;
 
 	    if (need > 0){
 		more_space(need);
-		strncpy(new_str + new_pos, str, need);
+		memmove(new_str + new_pos, str, need);
 		new_pos += need;
 	    }
 
@@ -753,7 +756,7 @@ int *range;
      *
      */
     more_space(str_len);
-    (void) strcpy(new_str + new_pos, str);
+    (void) memmove(new_str + new_pos, str, strlen(str) + 1);
     RETURN(new_str);
 }
 
