@@ -5,7 +5,7 @@ exec @TCLSH@ "$0" "$@"
 
 # Traverse through all ports, creating an index and archiving port directories
 # if requested
-# $Id: portindex.tcl 81366 2011-07-30 00:12:53Z jmr@macports.org $
+# $Id: portindex.tcl 96181 2012-08-02 14:33:15Z jmr@macports.org $
 
 source [file join "@macports_tcl_dir@" macports1.0 macports_fastload.tcl]
 package require macports
@@ -165,7 +165,7 @@ proc pindex {portdir} {
     }
 }
 
-if {[expr $argc > 4]} {
+if {[expr $argc > 8]} {
     print_usage
     exit 1
 }
@@ -187,6 +187,10 @@ for {set i 0} {$i < $argc} {incr i} {
                 set os_platform [lindex $platlist 0]
                 set os_major [lindex $platlist 1]
                 set os_arch [lindex $platlist 2]
+                if {$os_platform == "macosx"} {
+                    lappend port_options os.subplatform $os_platform os.universal_supported yes
+                    set os_platform darwin
+                }
                 lappend port_options os.platform $os_platform os.major $os_major os.arch $os_arch
             } elseif {$arg == "-f"} { # Completely rebuild index
                 set full_reindex 1
