@@ -1,8 +1,7 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
-# portconfigure.tcl
-# $Id: portconfigure.tcl 81564 2011-08-02 03:54:04Z jmr@macports.org $
+# $Id: portconfigure.tcl 109893 2013-08-21 14:37:40Z larryv@macports.org $
 #
-# Copyright (c) 2007 - 2011 The MacPorts Project
+# Copyright (c) 2007 - 2013 The MacPorts Project
 # Copyright (c) 2007 Markus W. Weissmann <mww@macports.org>
 # Copyright (c) 2002 - 2003 Apple Inc.
 # All rights reserved.
@@ -128,22 +127,30 @@ options configure.m32 configure.m64 configure.march configure.mtune
 default configure.march     {}
 default configure.mtune     {}
 # We could have debug/optimizations be global configurable at some point.
-options configure.optflags configure.cflags configure.cppflags configure.cxxflags configure.objcflags configure.ldflags configure.libs configure.fflags configure.f90flags configure.fcflags configure.classpath
-default configure.optflags  {-O2}
+options configure.optflags \
+        configure.cflags configure.cxxflags \
+        configure.objcflags configure.objcxxflags \
+        configure.cppflags configure.ldflags configure.libs \
+        configure.fflags configure.f90flags configure.fcflags \
+        configure.classpath
 # compiler flags section
-default configure.cflags    {${configure.optflags}}
-default configure.cppflags  {-I${prefix}/include}
-default configure.cxxflags  {${configure.optflags}}
-default configure.objcflags {${configure.optflags}}
-default configure.ldflags   {-L${prefix}/lib}
-default configure.libs      {}
-default configure.fflags    {${configure.optflags}}
-default configure.f90flags  {${configure.optflags}}
-default configure.fcflags   {${configure.optflags}}
-default configure.classpath {}
+default configure.optflags      {-Os}
+default configure.cflags        {${configure.optflags}}
+default configure.cxxflags      {${configure.optflags}}
+default configure.objcflags     {${configure.optflags}}
+default configure.objcxxflags   {${configure.optflags}}
+default configure.cppflags      {-I${prefix}/include}
+default configure.ldflags       {"-L${prefix}/lib -Wl,-headerpad_max_install_names"}
+default configure.libs          {}
+default configure.fflags        {${configure.optflags}}
+default configure.f90flags      {${configure.optflags}}
+default configure.fcflags       {${configure.optflags}}
+default configure.classpath     {}
 
 # tools section
-options configure.perl configure.python configure.ruby configure.install configure.awk configure.bison configure.pkg_config configure.pkg_config_path
+options configure.perl configure.python configure.ruby \
+        configure.install configure.awk configure.bison \
+        configure.pkg_config configure.pkg_config_path
 default configure.perl              {}
 default configure.python            {}
 default configure.ruby              {}
@@ -154,24 +161,32 @@ default configure.pkg_config        {}
 default configure.pkg_config_path   {}
 
 options configure.build_arch configure.ld_archflags configure.sdkroot
-default configure.build_arch {[portconfigure::choose_supported_archs ${build_arch}]}
-default configure.ld_archflags {[portconfigure::configure_get_ld_archflags]}
-default configure.sdkroot {[portconfigure::configure_get_sdkroot]}
-foreach tool {cc cxx objc f77 f90 fc} {
+default configure.build_arch    {[portconfigure::choose_supported_archs ${build_arch}]}
+default configure.ld_archflags  {[portconfigure::configure_get_ld_archflags]}
+default configure.sdkroot       {[portconfigure::configure_get_sdkroot]}
+foreach tool {cc cxx objc objcxx f77 f90 fc} {
     options configure.${tool}_archflags
     default configure.${tool}_archflags  "\[portconfigure::configure_get_archflags $tool\]"
 }
 
-options configure.universal_archs configure.universal_args configure.universal_cflags configure.universal_cppflags configure.universal_cxxflags configure.universal_ldflags
+options configure.universal_archs configure.universal_args \
+        configure.universal_cflags configure.universal_cxxflags \
+        configure.universal_objcflags configure.universal_objcxxflags \
+        configure.universal_cppflags configure.universal_ldflags
 default configure.universal_archs       {[portconfigure::choose_supported_archs ${universal_archs}]}
 default configure.universal_args        {--disable-dependency-tracking}
 default configure.universal_cflags      {[portconfigure::configure_get_universal_cflags]}
-default configure.universal_cppflags    {}
 default configure.universal_cxxflags    {[portconfigure::configure_get_universal_cflags]}
+default configure.universal_objcflags   {${configure.universal_cflags}}
+default configure.universal_objcxxflags {${configure.universal_cxxflags}}
+default configure.universal_cppflags    {}
 default configure.universal_ldflags     {[portconfigure::configure_get_universal_ldflags]}
 
 # Select a distinct compiler (C, C preprocessor, C++)
-options configure.ccache configure.distcc configure.pipe configure.cc configure.cxx configure.cpp configure.objc configure.f77 configure.f90 configure.fc configure.javac configure.compiler
+options configure.ccache configure.distcc configure.pipe configure.cc \
+        configure.cxx configure.cpp configure.objc configure.objcxx configure.f77 \
+        configure.f90 configure.fc configure.javac configure.compiler \
+        compiler.blacklist compiler.whitelist compiler.fallback
 default configure.ccache        {${configureccache}}
 default configure.distcc        {${configuredistcc}}
 default configure.pipe          {${configurepipe}}
@@ -179,56 +194,74 @@ default configure.cc            {[portconfigure::configure_get_compiler cc]}
 default configure.cxx           {[portconfigure::configure_get_compiler cxx]}
 default configure.cpp           {[portconfigure::configure_get_compiler cpp]}
 default configure.objc          {[portconfigure::configure_get_compiler objc]}
+default configure.objcxx        {[portconfigure::configure_get_compiler objcxx]}
 default configure.f77           {[portconfigure::configure_get_compiler f77]}
 default configure.f90           {[portconfigure::configure_get_compiler f90]}
 default configure.fc            {[portconfigure::configure_get_compiler fc]}
 default configure.javac         {[portconfigure::configure_get_compiler javac]}
 default configure.compiler      {[portconfigure::configure_get_default_compiler]}
+default compiler.fallback       {[portconfigure::get_compiler_fallback]}
+default compiler.blacklist      {}
+default compiler.whitelist      {}
 
 set_ui_prefix
 
 proc portconfigure::configure_start {args} {
-    global UI_PREFIX configure.compiler
-    
+    global UI_PREFIX
+
     ui_notice "$UI_PREFIX [format [msgcat::mc "Configuring %s"] [option subport]]"
 
-    set name ""
-    switch -exact ${configure.compiler} {
-        gcc { set name "System gcc" }
-        gcc-3.3 { set name "Mac OS X gcc 3.3" }
-        gcc-4.0 { set name "Mac OS X gcc 4.0" }
-        gcc-4.2 { set name "Mac OS X gcc 4.2" }
-        llvm-gcc-4.2 { set name "Mac OS X llvm-gcc 4.2" }
-        clang { set name "Mac OS X clang" }
-        apple-gcc-3.3 { set name "MacPorts Apple gcc 3.3" }
-        apple-gcc-4.0 { set name "MacPorts Apple gcc 4.0" }
-        apple-gcc-4.2 { set name "MacPorts Apple gcc 4.2" }
-        macports-gcc-4.0 { set name "MacPorts gcc 4.0" }
-        macports-gcc-4.1 { set name "MacPorts gcc 4.1" }
-        macports-gcc-4.2 { set name "MacPorts gcc 4.2" }
-        macports-gcc-4.3 { set name "MacPorts gcc 4.3" }
-        macports-gcc-4.4 { set name "MacPorts gcc 4.4" }
-        macports-gcc-4.5 { set name "MacPorts gcc 4.5" }
-        macports-gcc-4.6 { set name "MacPorts gcc 4.6" }
-        macports-llvm-gcc-4.2 { set name "MacPorts llvm-gcc 4.2" }
-        macports-clang { set name "MacPorts clang" }
-        default { return -code error "Invalid value for configure.compiler" }
+    set compiler [option configure.compiler]
+    set valid_compilers {
+        {^apple-gcc-(4\.[02])$}             {MacPorts Apple GCC %s}
+        {^cc$}                              {System cc}
+        {^clang$}                           {Xcode Clang}
+        {^gcc$}                             {System GCC}
+        {^gcc-(3\.3|4\.[02])$}              {Xcode GCC %s}
+        {^llvm-gcc-4\.2$}                   {Xcode LLVM-GCC 4.2}
+        {^macports-clang$}                  {MacPorts Clang (port select}
+        {^macports-clang-(\d+\.\d+)$}       {MacPorts Clang %s}
+        {^macports-dragonegg-(\d+\.\d+)$}   {MacPorts DragonEgg %s}
+        {^macports-dragonegg-(\d+\.\d+)-gcc-(\d+\.\d+)$}
+            {MacPorts DragonEgg %s with GCC %s}
+        {^macports-gcc$}                    {MacPorts GCC (port select)}
+        {^macports-gcc-(\d+\.\d+)$}         {MacPorts GCC %s}
+        {^macports-llvm-gcc-4\.2$}          {MacPorts LLVM-GCC 4.2}
     }
-    ui_debug "Using compiler '$name'"
+    foreach {re fmt} $valid_compilers {
+        if {[set matches [regexp -inline $re $compiler]] ne {}} {
+            set compiler_name [eval [linsert [lrange $matches 1 end] 0 format $fmt]]
+            break
+        }
+    }
+    if {![info exists compiler_name]} {
+        return -code error "Invalid value for configure.compiler: $compiler"
+    }
+    ui_debug "Using compiler '$compiler_name'"
 
     # Additional ccache directory setup
-    global configureccache ccache_dir ccache_size macportsuser
-    if {${configureccache}} {
+    global configure.ccache ccache_dir ccache_size macportsuser
+    if {${configure.ccache}} {
+        # Create ccache directory with correct permissions with root privileges
         elevateToRoot "configure ccache"
         if [catch {
                 file mkdir ${ccache_dir}
                 file attributes ${ccache_dir} -owner ${macportsuser} -permissions 0755
-                exec ccache -M ${ccache_size} >/dev/null
             } result] {
-            ui_warn "ccache_dir ${ccache_dir} could not be initialized; disabling ccache: $result"
-            set configureccache no
+            ui_warn "ccache_dir ${ccache_dir} could not be created; disabling ccache: $result"
+            set configure.ccache no
         }
         dropPrivileges
+
+        # Initialize ccache directory with the given maximum size
+        if {${configure.ccache}} {
+            if [catch {
+                exec ccache -M ${ccache_size} >/dev/null
+            } result] {
+                ui_warn "ccache_dir ${ccache_dir} could not be initialized; disabling ccache: $result"
+                set configure.ccache no
+            }
+        }
     }
 }
 
@@ -267,7 +300,9 @@ proc portconfigure::configure_get_archflags {tool} {
     } elseif {[tbool configure.m32]} {
         set flags "-m32"
     } elseif {${configure.build_arch} != ""} {
-        if {[arch_flag_supported] && ($tool == "cc" || $tool == "cxx" || $tool == "objc")} {
+        if {[arch_flag_supported ${configure.compiler}] &&
+            [regexp {^(?:cc|cxx|objc|objcxx)$} $tool]
+        } then {
             set flags "-arch ${configure.build_arch}"
         } elseif {${configure.build_arch} == "x86_64" || ${configure.build_arch} == "ppc64"} {
             set flags "-m64"
@@ -283,8 +318,8 @@ proc portconfigure::configure_get_archflags {tool} {
 # doesn't support -arch, because it could be used to link rather than using
 # ld directly. So we punt and let portfiles deal with that case.
 proc portconfigure::configure_get_ld_archflags {args} {
-    global configure.build_arch
-    if {${configure.build_arch} != "" && [arch_flag_supported]} {
+    global configure.build_arch configure.compiler
+    if {${configure.build_arch} != "" && [arch_flag_supported ${configure.compiler}]} {
         return "-arch ${configure.build_arch}"
     } else {
         return ""
@@ -292,13 +327,18 @@ proc portconfigure::configure_get_ld_archflags {args} {
 }
 
 proc portconfigure::configure_get_sdkroot {} {
-    global developer_dir macosx_deployment_target macosx_version os.arch os.platform
+    global developer_dir macosx_deployment_target macosx_version xcodeversion os.arch os.platform
     if {${os.platform} == "darwin" && ($macosx_deployment_target != $macosx_version
         || (${os.arch} == "powerpc" && $macosx_version == "10.4" && [variant_exists universal] && [variant_isset universal]))} {
-        if {$macosx_deployment_target == "10.4"} {
-            set sdk "${developer_dir}/SDKs/MacOSX10.4u.sdk"
+        if {[vercmp $xcodeversion 4.3] < 0} {
+            set sdks_dir "${developer_dir}/SDKs"
         } else {
-            set sdk "${developer_dir}/SDKs/MacOSX${macosx_deployment_target}.sdk"
+            set sdks_dir "${developer_dir}/Platforms/MacOSX.platform/Developer/SDKs"
+        }
+        if {$macosx_deployment_target == "10.4"} {
+            set sdk "${sdks_dir}/MacOSX10.4u.sdk"
+        } else {
+            set sdk "${sdks_dir}/MacOSX${macosx_deployment_target}.sdk"
         }
         if {[file exists $sdk]} {
             return $sdk
@@ -332,227 +372,293 @@ proc portconfigure::configure_get_universal_ldflags {args} {
 }
 
 # internal proc to determine if the compiler supports -arch
-proc portconfigure::arch_flag_supported {args} {
-    global configure.compiler
-    switch -exact ${configure.compiler} {
-        gcc-4.0 -
-        gcc-4.2 -
-        llvm-gcc-4.2 -
-        clang -
-        apple-gcc-4.0 -
-        apple-gcc-4.2 -
-        macports-clang {
-            return yes
-        }
-        default {
-            return no
+proc portconfigure::arch_flag_supported {compiler} {
+    return [regexp {^gcc-4|llvm|apple|clang} $compiler]
+}
+
+# maps compiler names to the port that provides them
+# TODO: Remove this after 2.2 is released and ports aren't referring to it.
+array set portconfigure::compiler_name_map {
+        apple-gcc-4.0           apple-gcc40
+        apple-gcc-4.2           apple-gcc42
+        macports-gcc-4.2        gcc42
+        macports-gcc-4.3        gcc43
+        macports-gcc-4.4        gcc44
+        macports-gcc-4.5        gcc45
+        macports-gcc-4.6        gcc46
+        macports-gcc-4.7        gcc47
+        macports-gcc-4.8        gcc48
+        macports-llvm-gcc-4.2   llvm-gcc42
+        macports-clang-2.9      clang-2.9
+        macports-clang-3.0      clang-3.0
+        macports-clang-3.1      clang-3.1
+        macports-clang-3.2      clang-3.2
+        macports-clang-3.3      clang-3.3
+        macports-dragonegg-3.0  dragonegg-3.0
+        macports-dragonegg-3.1  dragonegg-3.1
+        macports-dragonegg-3.2  dragonegg-3.2
+        macports-dragonegg-3.3  dragonegg-3.3
+}
+
+# Mapping from compiler names to compiler ports, for private use by
+# compiler_port_name. Do not access directly.
+set portconfigure::valid_compiler_ports {
+    {^apple-gcc-(\d+)\.(\d+)$}                          {apple-gcc%s%s}
+    {^macports-clang-(\d+\.\d+)$}                       {clang-%s}
+    {^macports-dragonegg-(\d+\.\d+)(-gcc-\d+\.\d+)?$}   {dragonegg-%s%s}
+    {^macports-(llvm-)?gcc-(\d+)\.(\d+)$}               {%sgcc%s%s}
+}
+
+proc portconfigure::compiler_port_name {compiler} {
+    foreach {re fmt} $portconfigure::valid_compiler_ports {
+        if {[set matches [regexp -inline $re $compiler]] ne {}} {
+            return [eval [linsert [lrange $matches 1 end] 0 format $fmt]]
         }
     }
+    return {}
+}
+
+proc portconfigure::compiler_is_port {compiler} {
+    return [expr {[portconfigure::compiler_port_name ${compiler}] != ""}]
 }
 
 # internal function to determine the default compiler
 proc portconfigure::configure_get_default_compiler {args} {
-    global xcodeversion macosx_deployment_target
-    if {$xcodeversion == "none" || $xcodeversion == ""} {
-        return gcc
-    } elseif {[rpm-vercomp $xcodeversion 4.2] >= 0} {
-        return clang
-    } elseif {[rpm-vercomp $xcodeversion 4.0] >= 0} {
-        return llvm-gcc-4.2
-    } elseif {[rpm-vercomp $xcodeversion 3.2] >= 0 && $macosx_deployment_target != "10.4"} {
-        return gcc-4.2
+    if {[option compiler.whitelist] != {}} {
+        set search_list [option compiler.whitelist]
     } else {
-        return gcc-4.0
+        set search_list [option compiler.fallback]
+    }
+    foreach compiler $search_list {
+        set allowed yes
+        foreach pattern [option compiler.blacklist] {
+            if {[string match $pattern $compiler]} {
+                set allowed no
+                break
+            }
+        }
+        if {$allowed &&
+            ([file executable [configure_get_compiler cc $compiler]] ||
+             [compiler_is_port $compiler])
+        } then {
+            return $compiler
+        }
+    }
+    ui_warn "All compilers are either blacklisted or unavailable; defaulting to first fallback option"
+    return [lindex [option compiler.fallback] 0]
+}
+
+# internal function to choose compiler fallback list based on platform
+proc portconfigure::get_compiler_fallback {} {
+    global xcodeversion macosx_deployment_target default_compilers
+    if {[info exists default_compilers]} {
+        return $default_compilers
+    } elseif {$xcodeversion == "none" || $xcodeversion == ""} {
+        return {cc}
+    } elseif {[vercmp $xcodeversion 4.7] >= 0} {
+        return {clang macports-llvm-gcc-4.2 apple-gcc-4.2 macports-clang-3.3}
+    } elseif {[vercmp $xcodeversion 4.2] >= 0} {
+        return {clang llvm-gcc-4.2 apple-gcc-4.2 macports-clang-3.3}
+    } elseif {[vercmp $xcodeversion 4.0] >= 0} {
+        return {llvm-gcc-4.2 clang gcc-4.2 macports-clang-3.3 apple-gcc-4.2}
+    } elseif {[vercmp $xcodeversion 3.2] >= 0} {
+        if {$macosx_deployment_target == "10.4"} {
+            # It's not the deployment target that is the issue, it's the
+            # 10.4u SDK which base chooses if the deployment_target is set
+            return {gcc-4.0}
+        } else {
+            return {gcc-4.2 clang llvm-gcc-4.2 macports-clang-3.3 macports-llvm-gcc-4.2 apple-gcc-4.2 gcc-4.0}
+        }
+    } elseif {[vercmp $xcodeversion 3.0] >= 0} {
+        return {gcc-4.2 apple-gcc-4.2 gcc-4.0 macports-clang-3.3}
+    } else {
+        return {gcc-4.0 apple-gcc-4.2 gcc-3.3 macports-clang-3.3}
     }
 }
 
+# Find a developer tool
+proc portconfigure::find_developer_tool {name} {
+	global developer_dir
+
+    # first try /usr/bin since this doesn't move around
+    set toolpath "/usr/bin/${name}"
+    if {[file executable $toolpath]} {
+        return $toolpath
+    }
+
+	# Use xcode's xcrun to find the named tool.
+	if {![catch {exec [findBinary xcrun $portutil::autoconf::xcrun_path] -find ${name}} toolpath]} {
+		return ${toolpath}
+	}
+
+	# If xcrun failed to find the tool, return a path from
+	# the developer_dir.
+	# The tool may not be there, but we'll leave it up to
+	# the invoking code to figure out that it doesn't have
+	# a valid compiler
+	return "${developer_dir}/usr/bin/${name}"
+}
+
+
 # internal function to find correct compilers
-proc portconfigure::configure_get_compiler {type} {
-    global configure.compiler prefix developer_dir
-    set ret ""
-    switch -exact ${configure.compiler} {
-        gcc {
-            switch -exact ${type} {
-                cc   { set ret /usr/bin/gcc }
-                objc { set ret /usr/bin/gcc }
-                cxx  { set ret /usr/bin/g++ }
-                cpp  { set ret /usr/bin/cpp }
-            }
-        }
-        gcc-3.3 {
-            switch -exact ${type} {
-                cc   { set ret /usr/bin/gcc-3.3 }
-                objc { set ret /usr/bin/gcc-3.3 }
-                cxx  { set ret /usr/bin/g++-3.3 }
-                cpp  { set ret /usr/bin/cpp-3.3 }
-            }
-        }
-        gcc-4.0 {
-            switch -exact ${type} {
-                cc   { set ret /usr/bin/gcc-4.0 }
-                objc { set ret /usr/bin/gcc-4.0 }
-                cxx  { set ret /usr/bin/g++-4.0 }
-                cpp  { set ret /usr/bin/cpp-4.0 }
-            }
-        }
-        gcc-4.2 {
-            switch -exact ${type} {
-                cc   { set ret /usr/bin/gcc-4.2 }
-                objc { set ret /usr/bin/gcc-4.2 }
-                cxx  { set ret /usr/bin/g++-4.2 }
-                cpp  { set ret /usr/bin/cpp-4.2 }
-            }
-        }
-        llvm-gcc-4.2 {
-            switch -exact ${type} {
-                cc   { set ret ${developer_dir}/usr/bin/llvm-gcc-4.2 }
-                objc { set ret ${developer_dir}/usr/bin/llvm-gcc-4.2 }
-                cxx  { set ret ${developer_dir}/usr/bin/llvm-g++-4.2 }
-                cpp  { set ret ${developer_dir}/usr/bin/llvm-cpp-4.2 }
-            }
-        }
-        clang {
-            switch -exact ${type} {
-                cc   { set ret ${developer_dir}/usr/bin/clang }
-                objc { set ret ${developer_dir}/usr/bin/clang }
-                cxx  {
-                    if {[file executable ${developer_dir}/usr/bin/clang++]} {
-                        set ret ${developer_dir}/usr/bin/clang++
-                    } else {
-                        set ret ${developer_dir}/usr/bin/llvm-g++-4.2
-                    }
+proc portconfigure::configure_get_compiler {type {compiler {}}} {
+    global configure.compiler prefix
+    if {$compiler == {}} {
+        set compiler ${configure.compiler}
+    }
+    # Tcl 8.4's switch doesn't support -matchvar.
+    if {[regexp {^apple-gcc(-4\.[02])$} $compiler -> suffix]} {
+        switch $type {
+            cc      -
+            objc    { return ${prefix}/bin/gcc-apple${suffix} }
+            cxx     -
+            objcxx  {
+                if {$suffix == "-4.2"} {
+                    return ${prefix}/bin/g++-apple${suffix}
                 }
             }
+            cpp     { return ${prefix}/bin/cpp-apple${suffix} }
         }
-        apple-gcc-3.3 {
-            switch -exact ${type} {
-                cc  { set ret ${prefix}/bin/gcc-apple-3.3 }
-                cpp { set ret ${prefix}/bin/cpp-apple-3.3 }
+    } elseif {[regexp {^clang$} $compiler]} {
+        switch $type {
+            cc      -
+            objc    { return [find_developer_tool clang] }
+            cxx     -
+            objcxx  {
+                set clangpp [find_developer_tool clang++]
+                if {[file executable $clangpp]} {
+                    return $clangpp
+                }
+                return [find_developer_tool llvm-g++-4.2]
             }
         }
-        apple-gcc-4.0 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-apple-4.0 }
-                objc { set ret ${prefix}/bin/gcc-apple-4.0 }
-                cpp  { set ret ${prefix}/bin/cpp-apple-4.0 }
-            }
+    } elseif {[regexp {^gcc(-3\.3|-4\.[02])?$} $compiler -> suffix]} {
+        switch $type {
+            cc      -
+            objc    { return [find_developer_tool "gcc${suffix}"] }
+            cxx     -
+            objcxx  { return [find_developer_tool "g++${suffix}"] }
+            cpp     { return [find_developer_tool "cpp${suffix}"] }
         }
-        apple-gcc-4.2 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-apple-4.2 }
-                objc { set ret ${prefix}/bin/gcc-apple-4.2 }
-                cpp  { set ret ${prefix}/bin/cpp-apple-4.2 }
-            }
+    } elseif {[regexp {^llvm-gcc-4\.2$} $compiler]} {
+        switch $type {
+            cc      -
+            objc    { return [find_developer_tool llvm-gcc-4.2] }
+            cxx     -
+            objcxx  { return [find_developer_tool llvm-g++-4.2] }
+            cpp     { return [find_developer_tool llvm-cpp-4.2] }
         }
-        macports-gcc-4.0 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-mp-4.0 }
-                objc { set ret ${prefix}/bin/gcc-mp-4.0 }
-                cxx  { set ret ${prefix}/bin/g++-mp-4.0 }
-                cpp  { set ret ${prefix}/bin/cpp-mp-4.0 }
-                fc   { set ret ${prefix}/bin/gfortran-mp-4.0 }
-                f77  { set ret ${prefix}/bin/gfortran-mp-4.0 }
-                f90  { set ret ${prefix}/bin/gfortran-mp-4.0 }
-            }
+    } elseif {[regexp {^macports-clang(-\d+\.\d+)?$} $compiler -> suffix]} {
+        if {$suffix ne {}} {
+            set suffix "-mp${suffix}"
         }
-        macports-gcc-4.1 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-mp-4.1 }
-                objc { set ret ${prefix}/bin/gcc-mp-4.1 }
-                cxx  { set ret ${prefix}/bin/g++-mp-4.1 }
-                cpp  { set ret ${prefix}/bin/cpp-mp-4.1 }
-                fc   { set ret ${prefix}/bin/gfortran-mp-4.1 }
-                f77  { set ret ${prefix}/bin/gfortran-mp-4.1 }
-                f90  { set ret ${prefix}/bin/gfortran-mp-4.1 }
-            }
+        switch $type {
+            cc      -
+            objc    { return ${prefix}/bin/clang${suffix} }
+            cxx     -
+            objcxx  { return ${prefix}/bin/clang++${suffix} }
         }
-        macports-gcc-4.2 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-mp-4.2 }
-                objc { set ret ${prefix}/bin/gcc-mp-4.2 }
-                cxx  { set ret ${prefix}/bin/g++-mp-4.2 }
-                cpp  { set ret ${prefix}/bin/cpp-mp-4.2 }
-                fc   { set ret ${prefix}/bin/gfortran-mp-4.2 }
-                f77  { set ret ${prefix}/bin/gfortran-mp-4.2 }
-                f90  { set ret ${prefix}/bin/gfortran-mp-4.2 }
-            }
+    } elseif {[regexp {^macports-dragonegg(-\d+\.\d+)(?:-gcc(-\d+\.\d+))?$} $compiler \
+                -> infix suffix]} {
+        if {$suffix ne {}} {
+            set suffix "-mp${suffix}"
         }
-        macports-gcc-4.3 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-mp-4.3 }
-                objc { set ret ${prefix}/bin/gcc-mp-4.3 }
-                cxx  { set ret ${prefix}/bin/g++-mp-4.3 }
-                cpp  { set ret ${prefix}/bin/cpp-mp-4.3 }
-                fc   { set ret ${prefix}/bin/gfortran-mp-4.3 }
-                f77  { set ret ${prefix}/bin/gfortran-mp-4.3 }
-                f90  { set ret ${prefix}/bin/gfortran-mp-4.3 }
-            }
+        switch $type {
+            cc      -
+            objc    { return ${prefix}/bin/dragonegg${infix}-gcc${suffix} }
+            cxx     -
+            objcxx  { return ${prefix}/bin/dragonegg${infix}-g++${suffix} }
+            cpp     { return ${prefix}/bin/dragonegg${infix}-cpp${suffix} }
+            fc      -
+            f77     -
+            f90     { return ${prefix}/bin/dragonegg${infix}-gfortran${suffix} }
         }
-        macports-gcc-4.4 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-mp-4.4 }
-                objc { set ret ${prefix}/bin/gcc-mp-4.4 }
-                cxx  { set ret ${prefix}/bin/g++-mp-4.4 }
-                cpp  { set ret ${prefix}/bin/cpp-mp-4.4 }
-                fc   { set ret ${prefix}/bin/gfortran-mp-4.4 }
-                f77  { set ret ${prefix}/bin/gfortran-mp-4.4 }
-                f90  { set ret ${prefix}/bin/gfortran-mp-4.4 }
-            }
+    } elseif {[regexp {^macports-gcc(-\d+\.\d+)?$} $compiler -> suffix]} {
+        if {$suffix ne {}} {
+            set suffix "-mp${suffix}"
         }
-        macports-gcc-4.5 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-mp-4.5 }
-                objc { set ret ${prefix}/bin/gcc-mp-4.5 }
-                cxx  { set ret ${prefix}/bin/g++-mp-4.5 }
-                cpp  { set ret ${prefix}/bin/cpp-mp-4.5 }
-                fc   { set ret ${prefix}/bin/gfortran-mp-4.5 }
-                f77  { set ret ${prefix}/bin/gfortran-mp-4.5 }
-                f90  { set ret ${prefix}/bin/gfortran-mp-4.5 }
-            }
+        switch $type {
+            cc      -
+            objc    { return ${prefix}/bin/gcc${suffix} }
+            cxx     -
+            objcxx  { return ${prefix}/bin/g++${suffix} }
+            cpp     { return ${prefix}/bin/cpp${suffix} }
+            fc      -
+            f77     -
+            f90     { return ${prefix}/bin/gfortran${suffix} }
         }
-        macports-gcc-4.6 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/gcc-mp-4.6 }
-                objc { set ret ${prefix}/bin/gcc-mp-4.6 }
-                cxx  { set ret ${prefix}/bin/g++-mp-4.6 }
-                cpp  { set ret ${prefix}/bin/cpp-mp-4.6 }
-                fc   { set ret ${prefix}/bin/gfortran-mp-4.6 }
-                f77  { set ret ${prefix}/bin/gfortran-mp-4.6 }
-                f90  { set ret ${prefix}/bin/gfortran-mp-4.6 }
-            }
-        }
-        macports-llvm-gcc-4.2 {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/llvm-gcc-4.2 }
-                objc { set ret ${prefix}/bin/llvm-gcc-4.2 }
-                cxx  { set ret ${prefix}/bin/llvm-g++-4.2 }
-                cpp  { set ret ${prefix}/bin/llvm-cpp-4.2 }
-                fc   { set ret ${prefix}/bin/llvm-gfortran-4.2 }
-                f77  { set ret ${prefix}/bin/llvm-gfortran-4.2 }
-                f90  { set ret ${prefix}/bin/llvm-gfortran-4.2 }
-            }
-        }
-        macports-clang {
-            switch -exact ${type} {
-                cc   { set ret ${prefix}/bin/clang }
-                objc { set ret ${prefix}/bin/clang }
-                cxx  { set ret ${prefix}/bin/clang++ }
-            }
+    } elseif {[regexp {^macports-llvm-gcc-4\.2$} $compiler]} {
+        switch $type {
+            cc      -
+            objc    { return ${prefix}/bin/llvm-gcc-4.2 }
+            cxx     -
+            objcxx  { return ${prefix}/bin/llvm-g++-4.2 }
+            cpp     { return ${prefix}/bin/llvm-cpp-4.2 }
         }
     }
-    return $ret
+    # Fallbacks
+    switch $type {
+        cc      -
+        objc    { return [find_developer_tool cc] }
+        cxx     -
+        objcxx  { return [find_developer_tool c++] }
+        cpp     { return [find_developer_tool cpp] }
+    }
+    return ""
 }
 
+# Automatically called from macports1.0 after evaluating the Portfile
+# Some of the compilers we use are provided by MacPorts itself; ensure we
+# automatically add a dependency when needed
+proc portconfigure::add_automatic_compiler_dependencies {} {
+    global configure.compiler configure.compiler.add_deps
+
+    if {!${configure.compiler.add_deps}} {
+        return
+    }
+
+    # The default value requires substitution before use.
+    set compiler [subst ${configure.compiler}]
+    if {![compiler_is_port $compiler]} {
+        return
+    }
+
+    ui_debug "Chosen compiler ${compiler} is provided by a port, adding dependency"
+
+    set compiler_port [portconfigure::compiler_port_name ${compiler}]
+    set deptype "build"
+    if {[string first "macports-gcc-" $compiler] == 0} {
+        set deptype "lib"
+    }
+    ui_debug "Adding depends_${deptype} port:$compiler_port"
+    depends_${deptype}-delete port:$compiler_port
+    depends_${deptype}-append port:$compiler_port
+
+    if {[arch_flag_supported $compiler]} {
+        ui_debug "Adding depends_skip_archcheck $compiler_port"
+        depends_skip_archcheck-delete $compiler_port
+        depends_skip_archcheck-append $compiler_port
+    }
+}
+# Register the above procedure as a callback after Portfile evaluation
+port::register_callback portconfigure::add_automatic_compiler_dependencies
+# and an option to turn it off if required
+options configure.compiler.add_deps
+default configure.compiler.add_deps yes
+
 proc portconfigure::configure_main {args} {
     global [info globals]
-    global worksrcpath use_configure use_autoreconf use_autoconf use_automake use_xmkmf
-    global configure.env configure.pipe configure.libs configure.classpath configure.universal_args
-    global configure.perl configure.python configure.ruby configure.install configure.awk configure.bison configure.pkg_config configure.pkg_config_path
-    global configure.ccache configure.distcc configure.cpp configure.javac configure.march configure.mtune configure.sdkroot
-    foreach tool {cc cxx objc f77 f90 fc ld} {
+    global worksrcpath use_configure use_autoreconf use_autoconf use_automake use_xmkmf \
+           configure.env configure.pipe configure.libs configure.classpath configure.universal_args \
+           configure.perl configure.python configure.ruby configure.install configure.awk configure.bison \
+           configure.pkg_config configure.pkg_config_path \
+           configure.ccache configure.distcc configure.cpp configure.javac configure.sdkroot \
+           configure.march configure.mtune \
+           os.platform os.major
+    foreach tool {cc cxx objc objcxx f77 f90 fc ld} {
         global configure.${tool} configure.${tool}_archflags
     }
-    foreach flags {cflags cppflags cxxflags objcflags ldflags fflags f90flags fcflags} {
+    foreach flags {cflags cppflags cxxflags objcflags objcxxflags ldflags fflags f90flags fcflags} {
         global configure.${flags} configure.universal_${flags}
     }
     
@@ -575,12 +681,16 @@ proc portconfigure::configure_main {args} {
     }
 
     if {[tbool use_xmkmf]} {
+        parse_environment xmkmf
+        append_list_to_environment_value xmkmf "IMAKECPP" ${configure.cpp}
         if {[catch {command_exec xmkmf} result]} {
             return -code error "[format [msgcat::mc "%s failure: %s"] xmkmf $result]"
-        } else {
-            # XXX should probably use make command abstraction but we know that
-            # X11 will already set things up so that "make Makefiles" always works.
-            system "cd ${worksrcpath} && make Makefiles"
+        }
+
+        parse_environment xmkmf
+        append_list_to_environment_value xmkmf "IMAKECPP" ${configure.cpp}
+        if {[catch {command_exec "cd ${worksrcpath} && make Makefiles" -varprefix xmkmf} result]} {
+            return -code error "[format [msgcat::mc "%s failure: %s"] "make Makefiles" $result]"
         }
     } elseif {[tbool use_configure]} {
         # Merge (ld|c|cpp|cxx)flags into the environment variable.
@@ -609,6 +719,7 @@ proc portconfigure::configure_main {args} {
         append_list_to_environment_value configure "CC" ${filter}${configure.cc}
         append_list_to_environment_value configure "CXX" ${filter}${configure.cxx}
         append_list_to_environment_value configure "OBJC" ${filter}${configure.objc}
+        append_list_to_environment_value configure "OBJCXX" ${filter}${configure.objcxx}
         append_list_to_environment_value configure "FC" ${configure.fc}
         append_list_to_environment_value configure "F77" ${configure.f77}
         append_list_to_environment_value configure "F90" ${configure.f90}
@@ -617,6 +728,7 @@ proc portconfigure::configure_main {args} {
         append_list_to_environment_value configure "CPPFLAGS" ${configure.cppflags}
         append_list_to_environment_value configure "CXXFLAGS" ${output}${configure.cxxflags}
         append_list_to_environment_value configure "OBJCFLAGS" ${output}${configure.objcflags}
+        append_list_to_environment_value configure "OBJCXXFLAGS" ${output}${configure.objcxxflags}
         append_list_to_environment_value configure "LDFLAGS" ${configure.ldflags}
         append_list_to_environment_value configure "LIBS" ${configure.libs}
         append_list_to_environment_value configure "FFLAGS" ${output}${configure.fflags}
@@ -632,9 +744,14 @@ proc portconfigure::configure_main {args} {
         append_list_to_environment_value configure "PKG_CONFIG" ${configure.pkg_config}
         append_list_to_environment_value configure "PKG_CONFIG_PATH" ${configure.pkg_config_path}
 
+        # https://trac.macports.org/ticket/34221
+        if {${os.platform} == "darwin" && ${os.major} == 12} {
+            append_list_to_environment_value configure "__CFPREFERENCES_AVOID_DAEMON" 1
+        }
+
         # add SDK flags if cross-compiling (or universal on ppc tiger)
         if {${configure.sdkroot} != ""} {
-            foreach flags {CPPFLAGS CFLAGS CXXFLAGS OBJCFLAGS} {
+            foreach flags {CPPFLAGS CFLAGS CXXFLAGS OBJCFLAGS OBJCXXFLAGS} {
                 append_list_to_environment_value configure $flags "-isysroot ${configure.sdkroot}"
             }
             append_list_to_environment_value configure "LDFLAGS" "-Wl,-syslibroot,${configure.sdkroot}"
@@ -642,15 +759,15 @@ proc portconfigure::configure_main {args} {
 
         # add extra flags that are conditional on whether we're building universal
         if {[variant_exists universal] && [variant_isset universal]} {
-            foreach flags {CFLAGS OBJCFLAGS} {
-                append_list_to_environment_value configure $flags ${configure.universal_cflags}
-            }
+            append_list_to_environment_value configure "CFLAGS" ${configure.universal_cflags}
             append_list_to_environment_value configure "CXXFLAGS" ${configure.universal_cxxflags}
+            append_list_to_environment_value configure "OBJCFLAGS" ${configure.universal_objcflags}
+            append_list_to_environment_value configure "OBJCXXFLAGS" ${configure.universal_objcxxflags}
             append_list_to_environment_value configure "CPPFLAGS" ${configure.universal_cppflags}
             append_list_to_environment_value configure "LDFLAGS" ${configure.universal_ldflags}
             eval configure.pre_args-append ${configure.universal_args}
         } else {
-            foreach {tool flags} {cc CFLAGS cxx CXXFLAGS objc OBJCFLAGS f77 FFLAGS f90 F90FLAGS fc FCFLAGS ld LDFLAGS} {
+            foreach {tool flags} {cc CFLAGS cxx CXXFLAGS objc OBJCFLAGS objcxx OBJCXXFLAGS f77 FFLAGS f90 F90FLAGS fc FCFLAGS ld LDFLAGS} {
                 append_list_to_environment_value configure $flags [set configure.${tool}_archflags]
                 if {${configure.march} != {}} {
                     append_list_to_environment_value configure $flags "-march=${configure.march}"
@@ -663,6 +780,10 @@ proc portconfigure::configure_main {args} {
 
         # Execute the command (with the new environment).
         if {[catch {command_exec configure} result]} {
+            global configure.dir
+            if {[file exists ${configure.dir}/config.log]} {
+                ui_error "[format [msgcat::mc "Failed to configure %s, consult %s/config.log"] [option subport] ${configure.dir}]"
+            }
             return -code error "[format [msgcat::mc "%s failure: %s"] configure $result]"
         }
     }
