@@ -1,6 +1,6 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:ft=tcl:et:sw=4:ts=4:sts=4
 # portdeactivate.tcl
-# $Id: portdeactivate.tcl 79597 2011-06-19 20:59:11Z jmr@macports.org $
+# $Id: portdeactivate.tcl 109245 2013-08-11 11:51:15Z cal@macports.org $
 #
 # Copyright (c) 2010-2011 The MacPorts Project
 # All rights reserved.
@@ -57,7 +57,15 @@ proc portdeactivate::deactivate_start {args} {
 }
 
 proc portdeactivate::deactivate_main {args} {
-    global subport version revision portvariants user_options
+    global subport version revision portvariants user_options startupitem.autostart UI_PREFIX
+
+    if {[tbool startupitem.autostart]} {
+        ui_notice "$UI_PREFIX [format [msgcat::mc "Unloading %s"] [option subport]]"
+        if {[eval_targets "unload"]} {
+            ui_warn [format [msgcat::mc "Failed to unload %s, continuing anyway."] [option subport]]
+        }
+    }
+
     registry_deactivate $subport $version $revision $portvariants [array get user_options]
     return 0
 }
