@@ -1,10 +1,11 @@
 # -*- coding: utf-8; mode: tcl; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- vim:fenc=utf-8:filetype=tcl:et:sw=4:ts=4:sts=4
 # portstartupitem.tcl
 #
-# $Id: portstartupitem.tcl 30823 2007-11-07 23:12:32Z afb@macports.org $
+# $Id: portstartupitem.tcl 109245 2013-08-11 11:51:15Z cal@macports.org $
 #
-# Copyright (c) 2004, 2005 Markus W. Weissman <mww@macports.org>,
-# Copyright (c) 2005 Robert Shaw <rshaw@opendarwin.org>,
+# Copyright (c) 2004-2012 The MacPorts Project
+# Copyright (c) 2006-2007 James D. Berry
+# Copyright (c) 2004,2005 Markus W. Weissman <mww@macports.org>
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -16,7 +17,7 @@
 # 2. Redistributions in binary form must reproduce the above copyright
 #    notice, this list of conditions and the following disclaimer in the
 #    documentation and/or other materials provided with the distribution.
-# 3. Neither the name of Apple Computer, Inc. nor the names of its
+# 3. Neither the name of The MacPorts Project nor the names of its
 #    contributors may be used to endorse or promote products derived from
 #    this software without specific prior written permission.
 # 
@@ -63,19 +64,25 @@
 #       - for launchd, generate log messages inside daemondo
 #       - for systemstarter, generate log messages in our generated script
 #
+#   startupitem.autoload    yes/no
+#       Automatically load the startupitem after activating. Defaults to no.
+#
 
 package provide portstartupitem 1.0
 package require portutil 1.0
 
+namespace eval portstartupitem {
+}
+
 set_ui_prefix
 
-proc startupitem_create_rcng {args} {
-    global prefix destroot portname os.platform
-    global startupitem.name startupitem.requires
-    global startupitem.start startupitem.stop startupitem.restart
-    global startupitem.type
+proc portstartupitem::startupitem_create_rcng {args} {
+    global prefix destroot os.platform \
+           startupitem.name startupitem.requires \
+           startupitem.start startupitem.stop startupitem.restart \
+           startupitem.type
 
-    set scriptdir ${destroot}/${prefix}/etc/rc.d
+    set scriptdir ${destroot}${prefix}/etc/rc.d
 
     if { ![exists startupitem.requires] } {
         set startupitem.requires ""
@@ -115,17 +122,21 @@ proc startupitem_create_rcng {args} {
     close ${fd}
 }
 
-proc startupitem_create_darwin_systemstarter {args} {
-    global UI_PREFIX prefix destroot destroot.keepdirs  portname os.platform
-    global startupitem.name startupitem.requires startupitem.init
-    global startupitem.start startupitem.stop startupitem.restart startupitem.executable
-    global startupitem.pidfile startupitem.logfile startupitem.logevents
+proc portstartupitem::startupitem_create_darwin_systemstarter {args} {
+    global UI_PREFIX prefix destroot destroot.keepdirs subport os.platform \
+           startupitem.name startupitem.requires startupitem.init \
+           startupitem.start startupitem.stop startupitem.restart startupitem.executable \
+           startupitem.pidfile startupitem.logfile startupitem.logevents
     
     set scriptdir ${prefix}/etc/startup
     
     set itemname            ${startupitem.name}
     set uppername           [string toupper ${startupitem.name}]
-    set itemdir             /Library/StartupItems/${itemname}
+    if {[getuid] == 0} {
+        set itemdir         /Library/StartupItems/${itemname}
+    } else {
+        set itemdir         ${prefix}/Library/StartupItems/${itemname}
+    }
     set startupItemDir      ${destroot}${itemdir}
     set startupItemScript   ${startupItemDir}/${itemname}
     set startupItemPlist    ${startupItemDir}/StartupParameters.plist
@@ -173,10 +184,10 @@ proc startupitem_create_darwin_systemstarter {args} {
         # An executable is specified, and there is no init, start, stop, or restart
     } else {
         if { ![llength ${startupitem.start} ] } {
-            set startupitem.start [list "sh ${scriptdir}/${portname}.sh start"]
+            set startupitem.start [list "sh ${scriptdir}/${subport}.sh start"]
         }
         if { ![llength ${startupitem.stop} ] } {
-            set startupitem.stop [list "sh ${scriptdir}/${portname}.sh stop"]
+            set startupitem.stop [list "sh ${scriptdir}/${subport}.sh stop"]
         }
     }
     if { ![llength ${startupitem.requires} ] } {
@@ -189,12 +200,16 @@ proc startupitem_create_darwin_systemstarter {args} {
     ########################
     # Create the startup item directory
     file mkdir ${startupItemDir}
-    file attributes ${startupItemDir} -owner root -group wheel
+    if {[getuid] == 0} {
+        file attributes ${startupItemDir} -owner root -group wheel
+    }
     
     ########################
     # Generate the startup item script
     set item [open "${startupItemScript}" w 0755]
-    file attributes "${startupItemScript}" -owner root -group wheel
+    if {[getuid] == 0} {
+        file attributes "${startupItemScript}" -owner root -group wheel
+    }
     
     # Emit the header
     puts ${item} {#!/bin/sh
@@ -358,7 +373,9 @@ RunService "$1"
     ########################
     # Generate the plist
     set para [open "${startupItemPlist}" w 0644]
-    file attributes "${startupItemPlist}" -owner root -group wheel
+    if {[getuid] == 0} {
+        file attributes "${startupItemPlist}" -owner root -group wheel
+    }
     
     puts ${para} "\{"
     puts ${para} "\tDescription\t= \"${itemname}\";"
@@ -371,28 +388,29 @@ RunService "$1"
     close ${para}
     
     # Emit some information for the user
-    ui_msg "###########################################################"
-    ui_msg "# A startup item has been generated that will aid in"
-    ui_msg "# starting ${portname} with SystemStarter. It is disabled"
-    ui_msg "# by default. Add the following line to /etc/hostconfig"
-    ui_msg "# or ${prefix}/etc/rc.conf to start it at startup:"
-    ui_msg "#"
-    ui_msg "# ${uppername}=-YES-"
-    ui_msg "###########################################################"
+    ui_notice "###########################################################"
+    ui_notice "# A startup item has been generated that will aid in"
+    ui_notice "# starting ${subport} with SystemStarter. It is disabled"
+    ui_notice "# by default. Add the following line to /etc/hostconfig"
+    ui_notice "# or ${prefix}/etc/rc.conf to start it at startup:"
+    ui_notice "#"
+    ui_notice "# ${uppername}=-YES-"
+    ui_notice "###########################################################"
 }
 
-proc startupitem_create_darwin_launchd {args} {
-    global UI_PREFIX prefix destroot destroot.keepdirs portname os.platform
-    global startupitem.name startupitem.requires startupitem.init
-    global startupitem.start startupitem.stop startupitem.restart startupitem.executable
-    global startupitem.pidfile startupitem.logfile startupitem.logevents startupitem.netchange
+proc portstartupitem::startupitem_create_darwin_launchd {args} {
+    global UI_PREFIX prefix destroot destroot.keepdirs subport macosx_deployment_target \
+           startupitem.name startupitem.uniquename startupitem.plist startupitem.location \
+           startupitem.init startupitem.start startupitem.stop startupitem.restart startupitem.executable \
+           startupitem.pidfile startupitem.logfile startupitem.logevents startupitem.netchange \
+           startupitem.install
 
     set scriptdir ${prefix}/etc/startup
     
     set itemname        ${startupitem.name}
-    set uniquename      org.macports.${itemname}
-    set plistname       ${uniquename}.plist
-    set daemondest      LaunchDaemons
+    set uniquename      ${startupitem.uniquename}
+    set plistname       ${startupitem.plist}
+    set daemondest      ${startupitem.location}
     set itemdir         ${prefix}/etc/${daemondest}/${uniquename}
     set args            [list \
                           "${prefix}/bin/daemondo" \
@@ -400,7 +418,9 @@ proc startupitem_create_darwin_launchd {args} {
                         ]
     
     file mkdir ${destroot}${itemdir}
-    file attributes ${destroot}${itemdir} -owner root -group wheel
+    if {[getuid] == 0} {
+        file attributes ${destroot}${itemdir} -owner root -group wheel
+    }
         
     if { [llength ${startupitem.executable}] && 
       ![llength ${startupitem.init}] &&
@@ -421,10 +441,10 @@ proc startupitem_create_darwin_launchd {args} {
         set wrapper         "${itemdir}/${wrappername}"
 
         if { ![llength ${startupitem.start}] } {
-            set startupitem.start [list "sh ${scriptdir}/${portname}.sh start"]
+            set startupitem.start [list "sh ${scriptdir}/${subport}.sh start"]
         }
         if { ![llength ${startupitem.stop}] } {
-            set startupitem.stop [list "sh ${scriptdir}/${portname}.sh stop"]
+            set startupitem.stop [list "sh ${scriptdir}/${subport}.sh stop"]
         }
         if { ![llength ${startupitem.restart}] } {
             set startupitem.restart [list Stop Start]
@@ -437,7 +457,9 @@ proc startupitem_create_darwin_launchd {args} {
 
         # Create the wrapper script
         set item [open "${destroot}${wrapper}" w 0755]
-        file attributes "${destroot}${wrapper}" -owner root -group wheel
+        if {[getuid] == 0} {
+            file attributes "${destroot}${wrapper}" -owner root -group wheel
+        }
 
         puts ${item} "#!/bin/sh"
         puts ${item} "#"
@@ -555,8 +577,8 @@ proc startupitem_create_darwin_launchd {args} {
     set plist [open "${destroot}${itemdir}/${plistname}" w 0644]
     
     puts ${plist} "<?xml version='1.0' encoding='UTF-8'?>"
-    puts ${plist} "<!DOCTYPE plist PUBLIC -//Apple Computer//DTD PLIST 1.0//EN"
-    puts ${plist} "http://www.apple.com/DTDs/PropertyList-1.0.dtd >"
+    puts ${plist} "<!DOCTYPE plist PUBLIC \"-//Apple//DTD PLIST 1.0//EN\""
+    puts ${plist} "\"http://www.apple.com/DTDs/PropertyList-1.0.dtd\" >"
     puts ${plist} "<plist version='1.0'>"
     puts ${plist} "<dict>"
     
@@ -569,8 +591,11 @@ proc startupitem_create_darwin_launchd {args} {
     
     puts ${plist} "<key>Debug</key><false/>"
     puts ${plist} "<key>Disabled</key><true/>"
-    puts ${plist} "<key>OnDemand</key><false/>"
-    puts ${plist} "<key>RunAtLoad</key><false/>"
+    if {$macosx_deployment_target != "10.4"} {
+        puts ${plist} "<key>KeepAlive</key><true/>"
+    } else {
+        puts ${plist} "<key>OnDemand</key><false/>"
+    }
     
     if { [llength ${startupitem.logfile}] } {
         puts ${plist} "<key>StandardOutPath</key><string>${startupitem.logfile}</string>"
@@ -581,35 +606,36 @@ proc startupitem_create_darwin_launchd {args} {
 
     close ${plist}
 
-    # Make a symlink to the plist file
-    file mkdir "${destroot}/Library/${daemondest}"
-    system "cd ${destroot}/Library/${daemondest} && ln -sf ${itemdir}/${plistname}"
-    
+    if { [getuid] == 0 && 
+      ${startupitem.install} != "no" } {
+        file mkdir "${destroot}/Library/${daemondest}"
+        ln -sf "${itemdir}/${plistname}" "${destroot}/Library/${daemondest}"
+    }
+
     # If launchd is not available, warn the user
     set haveLaunchd ${portutil::autoconf::have_launchd}
     if {![tbool haveLaunchd]} {
-        ui_msg "###########################################################"
-        ui_msg "# WARNING:"
-        ui_msg "# We're building a launchd startup item, but launchd wasn't"
-        ui_msg "# found by configure. Are you sure you didn't mess up your"
-        ui_msg "# ports.conf settings?"
-        ui_msg "###########################################################"
+        ui_notice "###########################################################"
+        ui_notice "# WARNING:"
+        ui_notice "# We're building a launchd startup item, but launchd wasn't"
+        ui_notice "# found by configure. Are you sure you didn't mess up your"
+        ui_notice "# macports.conf settings?"
+        ui_notice "###########################################################"
     }
     
     # Emit some information for the user
-    ui_msg "###########################################################"
-    ui_msg "# A startup item has been generated that will aid in"
-    ui_msg "# starting ${portname} with launchd. It is disabled"
-    ui_msg "# by default. Execute the following command to start it,"
-    ui_msg "# and to cause it to launch at startup:"
-    ui_msg "#"
-    ui_msg "# sudo launchctl load -w /Library/${daemondest}/${plistname}"
-    ui_msg "###########################################################"
+    ui_notice "###########################################################"
+    ui_notice "# A startup item has been generated that will aid in"
+    ui_notice "# starting ${subport} with launchd. It is disabled"
+    ui_notice "# by default. Execute the following command to start it,"
+    ui_notice "# and to cause it to launch at startup:"
+    ui_notice "#"
+    ui_notice "# sudo port load ${subport}"
+    ui_notice "###########################################################"
 }
 
-proc startupitem_create {args} {
-    global UI_PREFIX
-    global startupitem.type os.platform
+proc portstartupitem::startupitem_create {args} {
+    global UI_PREFIX startupitem.type os.platform
     
     set startupitem.type [string tolower ${startupitem.type}]
     
@@ -631,9 +657,9 @@ proc startupitem_create {args} {
     }
 
     if { ${startupitem.type} == "none" } {
-        ui_msg "$UI_PREFIX [msgcat::mc "Skipping creation of control script"]"
+        ui_notice "$UI_PREFIX [msgcat::mc "Skipping creation of control script"]"
     } else {
-        ui_msg "$UI_PREFIX [msgcat::mc "Creating ${startupitem.type} control script"]"
+        ui_notice "$UI_PREFIX [msgcat::mc "Creating ${startupitem.type} control script"]"
 
         switch -- ${startupitem.type} {
             launchd         { startupitem_create_darwin_launchd }
